@@ -15,13 +15,13 @@ setGeneric(name = "predict")
 # Steps performed by all models but different between base (no cov) and covariate models
 
 # . Estimate ---------------------------------------------------------------------------------------------------
-setGeneric("clv.controlflow.estimate.check.inputs", def=function(clv.fitted,  start.params.model, use.cor, start.param.cor, optimx.args, verbose,...)
+setGeneric("clv.controlflow.estimate.check.inputs", def=function(clv.fitted,  start.params.model, optimx.args, verbose,...)
   standardGeneric("clv.controlflow.estimate.check.inputs"))
 
-setGeneric("clv.controlflow.estimate.put.inputs", def=function(clv.fitted, cl, use.cor, ...)
+setGeneric("clv.controlflow.estimate.put.inputs", def=function(clv.fitted, verbose, ...)
   standardGeneric("clv.controlflow.estimate.put.inputs"))
 
-setGeneric("clv.controlflow.estimate.generate.start.params", def=function(clv.fitted, start.params.model,start.param.cor,verbose,...)
+setGeneric("clv.controlflow.estimate.generate.start.params", def=function(clv.fitted, start.params.model, verbose,...)
   standardGeneric("clv.controlflow.estimate.generate.start.params"))
 
 setGeneric("clv.controlflow.estimate.prepare.optimx.args", def=function(clv.fitted, start.params.all)
@@ -32,20 +32,39 @@ setGeneric("clv.controlflow.estimate.process.post.estimation", def=function(clv.
 
 
 # . Predict -----------------------------------------------------------------------------------------------
-setGeneric("clv.controlflow.predict.check.inputs", def = function(clv.fitted, prediction.end, continuous.discount.factor, predict.spending, verbose)
+setGeneric("clv.controlflow.predict.check.inputs", def = function(clv.fitted, verbose, ...)
   standardGeneric("clv.controlflow.predict.check.inputs"))
 
 setGeneric("clv.controlflow.predict.set.prediction.params", def = function(clv.fitted)
   standardGeneric("clv.controlflow.predict.set.prediction.params"))
 
+setGeneric("clv.controlflow.predict.build.result.table", def = function(clv.fitted, verbose, ...)
+  standardGeneric("clv.controlflow.predict.build.result.table"))
+
+setGeneric(name = "clv.controlflow.predict.get.has.actuals", def = function(clv.fitted, dt.predictions)
+  standardGeneric("clv.controlflow.predict.get.has.actuals"))
+
+setGeneric(name = "clv.controlflow.predict.add.actuals", def = function(clv.fitted, dt.predictions, has.actuals, verbose, ...)
+  standardGeneric("clv.controlflow.predict.add.actuals"))
+
+setGeneric(name = "clv.controlflow.predict.post.process.prediction.table", def = function(clv.fitted, dt.predictions, has.actuals, verbose, ...)
+  standardGeneric("clv.controlflow.predict.post.process.prediction.table"))
+
+
+
 # .. Newdata: replace data in existing model -----------------------------------------------------------------
 # For plot and predict
-setGeneric("clv.controlflow.check.newdata", def = function(clv.fitted, user.newdata, prediction.end)
+setGeneric("clv.controlflow.check.newdata", def = function(clv.fitted, user.newdata, ...)
   standardGeneric("clv.controlflow.check.newdata"))
 
+# .. Prediction params -------------------------------------------------------------------------------------
+# Check whether prediction params are ok to predict/plot
+setGeneric("clv.controlflow.check.prediction.params", def = function(clv.fitted)
+  standardGeneric("clv.controlflow.check.prediction.params"))
+
 
 # . Plot ----------------------------------------------------------------------------------------------------
-#function(clv.fitted, prediction.end, data.dyn.cov.life, data.dyn.cov.trans,...)
+# clv.controlflow.plot.check.inputs is needed for fitted.dyncov models only to check dyncov length
 setGeneric("clv.controlflow.plot.check.inputs", def = function(obj, prediction.end, cumulative, plot, label.line, verbose)
   standardGeneric("clv.controlflow.plot.check.inputs"))
 
@@ -62,15 +81,18 @@ setGeneric("clv.controlflow.plot.get.data", def = function(obj, dt.expectation.s
 
 # .. Estimate ----------------------------------------------------------------------------------------------------------------------
 # Perform model specific checks on user inputs to estimate
-setGeneric("clv.model.check.input.args", def = function(clv.model, clv.fitted, start.params.model, use.cor, start.param.cor, optimx.args, verbose, ...)
+setGeneric("clv.model.check.input.args", def = function(clv.model, clv.fitted, start.params.model, optimx.args, verbose, ...)
   standardGeneric("clv.model.check.input.args"))
 
 # Store additional arguments potentially given in estimate
-setGeneric("clv.model.put.estimation.input", def = function(clv.model, clv.fitted, verbose, ...)
+setGeneric("clv.model.put.estimation.input", def = function(clv.model, ...)
   standardGeneric("clv.model.put.estimation.input"))
 
+setGeneric(name="clv.model.generate.start.param.cor", def = function(clv.model, start.param.cor, transformed.start.params.model)
+  standardGeneric("clv.model.generate.start.param.cor"))
+
 # Finish the arguments to optimx with model specific arguments (mostly LL)
-setGeneric(name="clv.model.prepare.optimx.args", def=function(clv.model, clv.fitted, prepared.optimx.args,...)
+setGeneric(name="clv.model.prepare.optimx.args", def=function(clv.model, clv.fitted, prepared.optimx.args)
   standardGeneric("clv.model.prepare.optimx.args"))
 
 # Transform standard or user given start params to optimizer (prefixed) scale
@@ -85,6 +107,24 @@ setGeneric(name="clv.model.backtransform.estimated.params.model", def=function(c
 setGeneric(name="clv.model.process.post.estimation", def=function(clv.model, clv.fitted, res.optimx)
   standardGeneric("clv.model.process.post.estimation"))
 
+# . Density for spending models
+setGeneric(name="clv.model.probability.density", def=function(clv.model, x, clv.fitted)
+  standardGeneric("clv.model.probability.density"))
+
+
+# .. Correlation ---------------------------------------------------------------------------------
+# Whether the model in general supports life/trans correlation
+setGeneric(name="clv.model.supports.correlation", def = function(clv.model)
+  standardGeneric("clv.model.supports.correlation"))
+
+# Whether this fit used correlation
+setGeneric(name="clv.model.estimation.used.correlation", def = function(clv.model)
+  standardGeneric("clv.model.estimation.used.correlation"))
+
+# Adds the correlation parameter to a given vector of params after reading it from the optimx results
+setGeneric(name="clv.model.coef.add.correlation", def = function(clv.model, last.row.optimx.coef, original.scale.params)
+  standardGeneric("clv.model.coef.add.correlation"))
+
 setGeneric(name="clv.model.m.to.cor", def = function(clv.model, prefixed.params.model, param.m)
   standardGeneric("clv.model.m.to.cor"))
 
@@ -93,9 +133,8 @@ setGeneric(name="clv.model.cor.to.m", def = function(clv.model, prefixed.params.
 
 # .. Predict ----------------------------------------------------------------------------------------------------------------------
 # Predict clv per model
-
-setGeneric(name = "clv.model.predict.clv", def = function(clv.model, clv.fitted, dt.prediction, continuous.discount.factor, verbose)
-  standardGeneric("clv.model.predict.clv"))
+setGeneric(name = "clv.model.predict", def = function(clv.model, clv.fitted, dt.predictions, verbose, ...)
+  standardGeneric("clv.model.predict"))
 
 setGeneric(name = "clv.model.expectation", def = function(clv.model, clv.fitted, dt.expectation.seq, verbose)
   standardGeneric("clv.model.expectation"))
@@ -107,8 +146,8 @@ setGeneric(name="clv.model.vcov.jacobi.diag", def=function(clv.model, clv.fitted
 
 # .. Newdata ---------------------------------------------------------------------------------------------------------------
 # Do the steps necessary to integrate user newdata in the fitted model (ie do cbs etc)
-setGeneric(name="clv.model.put.newdata", def=function(clv.model, clv.fitted, user.newdata, verbose)
-  standardGeneric("clv.model.put.newdata"))
+setGeneric(name="clv.model.process.newdata", def=function(clv.model, clv.fitted, user.newdata, verbose)
+  standardGeneric("clv.model.process.newdata"))
 
 
 
